require "socket"
require "date"
require "rmail"

# load AR modules
require "rubygems" unless File.directory?("web/vendor/rails")
require "active_record"
RAILS_ENV = ENV["RAILS_ENV"] || "development"
ActiveRecord::Base.configurations = YAML::load(File.open("web/config/database.yml"))
ActiveRecord::Base.establish_connection

require "web/app/models/user"
require "web/app/models/mail"

class SmtpServer < TCPServer
  def initialize(server = nil)
    super("", 25)
    @tg = ThreadGroup.new
  end
  def save_mail(recv, data)
    mail = Mail.new
    message = RMail::Parser.read(data.gsub(/\r\n/, "\n"))
    mail.sender = message.header["From"]
    mail.subject = message.header["Subject"]
    mail.date = DateTime::parse(message.header["Date"])
    if message.multipart?
      mail.content = message.part(0).body
    else
      mail.content = message.body
    end
    recv.each do |reciver|
      mail.reciver = reciver
      mail.save
    end
  end
  def process(s)
    data = ""
    recv = []
    capture = false
    s.puts "220 #{ @server } Ready"
    while (line = s.gets)
      if capture
        if line[0] == 46
          save_mail recv, data
          capture = false
          s.puts "250 OK"
        else
          data += line
        end
      else
        cmd, value = line.split(" ", 2)
        case cmd
        when "EHLO"
          s.puts "250 #{ @server } hello #{ value }"
          s.puts "250 SIZE 10485760"
        when "RCPT"
          begin
            recv.push(User.find_first(["login = ?", /TO:<(.+?)@.+>/.match(value)[1]]).id)
          rescue
            # ignore
          end
          s.puts "250 OK"
        when "MAIL"
          s.puts "250 OK"
        when "DATA"
          capture = true
        when "QUIT"
          s.puts "221 #{ @server } Closing"
        else
          s.puts "250 OK"
        end
      end
    end 
  end
  def run
    listen 5
    while (s = accept)
      @tg.add( Thread.new { process s } )
    end
  end
end

SmtpServer.new.run
